@@ -3,7 +3,7 @@ from .utilities import FailedValidationError
 from ..canmsgs import MessageType
 
 
-def parse(cob_id: int, data: bytes, eds_config: EDS):
+def parse(cob_id: int, data: list, eds_config: EDS):
     """
     Parse Heartbeat message
 
@@ -22,14 +22,7 @@ def parse(cob_id: int, data: bytes, eds_config: EDS):
         0x7F: "Pre-operational"
     }
     node_id = MessageType.cob_id_to_node_id(cob_id)
-    hex_data = int(hex(data[0]), 16)
-    state = states.get(hex_data)
-
-    if state is None:
-        return "Invalid State"
-    else:
-        if int.from_bytes(data, "big") in states:
-            return state
-        else:
-            raise FailedValidationError(data, node_id, cob_id, __name__,
-                                        "Invalid heartbeat state detected")
+    if len(data) < 1 or data[0] not in states:
+        raise FailedValidationError(data, node_id, cob_id, __name__,
+                                    "Invalid heartbeat state detected")
+    return states.get(data[0])
