@@ -3,6 +3,7 @@ from unittest.mock import mock_open, patch
 
 from canopen_monitor.parser import eds
 from canopen_monitor.parser.hb import parse
+from canopen_monitor.parser.utilities import FailedValidationError
 from tests import TEST_EDS
 
 
@@ -19,7 +20,7 @@ class TestHB(unittest.TestCase):
         """
         Test Heartbeat Message
         """
-        hb_message = b'\x04'
+        hb_message = [0x04]
         self.assertEqual("Stopped",
                          parse(123, hb_message, self.eds),
                          "Error on heartbeat Message parse")
@@ -28,10 +29,23 @@ class TestHB(unittest.TestCase):
         """
         Test Heartbeat Message with an invalid payload
         """
-        hb_message = b'\xFF'
-        self.assertEqual("Invalid State",
-                         parse(123, hb_message, self.eds),
-                         "Error on heartbeat Message parse")
+        hb_message = [0xFF]
+        with self.assertRaises(FailedValidationError) as context:
+            parse(123, hb_message, self.eds)
+
+        self.assertEqual("Invalid heartbeat state detected",
+                          str(context.exception))
+
+    def test_HB_Empty(self):
+        """
+        Test Heartbeat Message with an invalid payload
+        """
+        hb_message = []
+        with self.assertRaises(FailedValidationError) as context:
+            parse(123, hb_message, self.eds)
+
+        self.assertEqual("Invalid heartbeat state detected",
+        str(context.exception))
 
 
 if __name__ == '__main__':
